﻿using Digst.OioIdws.OioWsTrust;
using Digst.OioIdws.Rest.Client;
using Digst.OioIdws.Wsc.OioWsTrust;
using log4net.Config;
using System;
using System.IdentityModel.Tokens;
using System.Net;
using System.Net.Http;
using System.Threading;

namespace Digst.OioIdws.WscExample {
    public class Program {
        static void Main(string[] args) {

            // Setup Log4Net configuration by loading it from configuration file. 
            // log4net is not necessary and is only being used for demonstration.
            XmlConfigurator.Configure();

            // To ensure that the WSP is up and running. (Video api)
            // Thread.Sleep(1000);
            ServicePointManager.SecurityProtocol = SecurityProtocolType.Tls12;

            // Retrieve token
            var config = TokenServiceConfigurationFactory.CreateConfiguration();
            IStsTokenService stsTokenService = new StsTokenServiceCache(config); //does both STS + WSP auth
            SecurityToken securityToken = stsTokenService.GetToken();

            var idwsClient = new OioIdwsClient(new OioIdwsClientSettings {
                ClientCertificate = config.ClientCertificate,
                CacheClockSkew = config.CacheClockSkew,
                SecurityTokenService = new OioIdwsStsSettings {
                    Certificate = config.StsCertificate,
                    EndpointAddress = new Uri(config.StsEndpointAddress),
                    TokenLifeTime = TimeSpan.FromMinutes(config.TokenLifeTimeInMinutes ?? 8 * 60)
                },
                AudienceUri = new Uri("urn:medcom:videoapi"),
                UseTokenCache = true,
                AccessTokenIssuerEndpoint = new Uri("https://videoapi.test-vdxapi.vconf.dk/videoapi/token"),
                DesiredAccessTokenExpiry = null
            }, securityToken);

            try {
                var httpClient = new HttpClient(idwsClient.CreateMessageHandler());

                {
                    //first invocation - security token is retrieved and stored in the AS, access token cached by client
                    var response = httpClient.GetAsync("https://videoapi.test-vdxapi.vconf.dk/videoapi/info");
                    response.Wait();
                    var responseString = response.Result.EnsureSuccessStatusCode().Content.ReadAsStringAsync();
                    responseString.Wait();
                    Console.WriteLine(responseString.Result);
                }
            } catch (Exception ex) {
                Console.Write("...so sad" + ex.Message);
            }

            Console.ReadKey();
        }
    }

}